@@ -1,10 +1,14 @@
+from os import path
+
 import geopandas as gpd
-
 import pandas as pd
-
 import spatialpandas
+import datashader as ds
 
 from mapshader.colors import colors
+from mapshader.io import load_raster
+from mapshader.transforms import reproject_raster
+from mapshader.transforms import reproject_vector
 
 
 class MapSource():
@@ -23,6 +27,9 @@ class MapSource():
         if extras is None:
             extras = []
 
+        if span == 'min/max' and zfield is None and geometry_type != 'raster':
+            raise ValueError('You must include a zfield for min/max scan calculation')
+
         self.name = name
         self.df = df
         self.data = data
@@ -57,6 +64,12 @@ class MapSource():
                '/<width>/<height>')
         return url
 
+    @property
+    def wms_url(self):
+        url = (f'/{self.key}'
+               '/wms')
+        return url
+
     @property
     def geojson_url(self):
         url = (f'/{self.key}'
@@ -80,11 +93,12 @@ def world_countries_source():
     data = gpd.datasets.get_path('naturalearth_lowres')
     world = gpd.read_file(data)
     world = world[(world.name != "Antarctica") & (world.name != "Fr. S. Antarctic Lands")]
-    world = world.to_crs(epsg=3857)
+    world = reproject_vector(world)
     spgdf = spatialpandas.GeoDataFrame(world, geometry='geometry')
     return MapSource(name='World Countries',
                      geometry_type='polygon',
                      df=spgdf,
+                     span='min/max',
                      xfield='geometry',
                      yfield='geometry',
                      zfield='pop_est',
@@ -95,15 +109,46 @@ def world_countries_source():
                      text='World Countries')
 
 
+def world_boundaries_source():
+
+    from mapshader.transforms import line_geoseries_to_datashader_line
+
+    data = gpd.datasets.get_path('naturalearth_lowres')
+    world = gpd.read_file(data)
+    world = world[(world.name != "Antarctica") & (world.name != "Fr. S. Antarctic Lands")]
+    world = reproject_vector(world)
+
+    # convert polys to datashader compatible lines
+    world['geometry'] = world['geometry'].exterior
+
+    line_df = line_geoseries_to_datashader_line(world['geometry'])
+
+    if not len(world) == len(line_df):
+        print('WARNING: dropping records during line conversion')
+
+    return MapSource(name='World Boundaries',
+                     geometry_type='line',
+                     df=line_df,
+                     cmap=['black', 'black'],
+                     xfield='x',
+                     yfield='y',
+                     agg_func='max',
+                     dynspread=2,
+                     shade_how='linear',
+                     key='world-boundaries',
+                     text='World Boundaries')
+
+
 def world_cities_source():
     data = gpd.datasets.get_path('naturalearth_cities')
     gdf = gpd.read_file(data)
-    gdf = gdf.to_crs(epsg=3857)
+    gdf = reproject_vector(gdf)
     gdf['X'] = gdf.geometry.apply(lambda p: p.x)
     gdf['Y'] = gdf.geometry.apply(lambda p: p.y)
     spgdf = spatialpandas.GeoDataFrame(gdf, geometry='geometry')
     return MapSource(name='World Cities',
                      geometry_type='point',
+                     cmap=['black', 'black'],
                      df=spgdf,
                      xfield='X',
                      yfield='Y',
@@ -116,7 +161,7 @@ def world_cities_source():
 def nybb_source():
     data = gpd.datasets.get_path('nybb')
     gdf = gpd.read_file(data)
-    gdf = gdf.to_crs(epsg=3857)
+    gdf = reproject_vector(gdf)
     spgdf = spatialpandas.GeoDataFrame(gdf, geometry='geometry')
     return MapSource(name='NYC Admin',
                      df=spgdf,
@@ -124,6 +169,7 @@ def nybb_source():
                      yfield='geometry',
                      zfield='BoroCode',
                      agg_func='max',
+                     span='min/max',
                      dynspread=None,
                      shade_how='linear',
                      geometry_type='polygon',
@@ -131,6 +177,28 @@ def nybb_source():
                      text='NYC Admin')
 
 
+def elevation_source():
+    HERE = path.abspath(path.dirname(__file__))
+    FIXTURES_DIR = path.join(HERE, 'tests', 'fixtures')
+    elevation_path = path.join(FIXTURES_DIR, 'elevation.tif')
+    arr = load_raster(elevation_path)
+    arr = arr.squeeze().drop("band")
+    arr.data = arr.data.astype('f8')
+    arr.data = ds.utils.orient_array(arr)
+    arr = arr.assign_coords(y=list(reversed(arr.coords['y'])))
+    arr = reproject_raster(arr)
+    return MapSource(name='Elevation',
+                     df=arr,
+                     xfield='geometry',
+                     yfield='geometry',
+                     raster_interpolate='linear',
+                     shade_how='linear',
+                     span='min/max',
+                     geometry_type='raster',
+                     key='elevation',
+                     text='Elevation')
+
+
 def get_user_datasets() -> dict:
     return {}
 
@@ -138,8 +206,10 @@ def get_user_datasets() -> dict:
 # TODO: Add default line and raster datasets
 default_datasets = {
     'world-countries': world_countries_source(),
+    'world-boundaries': world_boundaries_source(),
     'world-cities': world_cities_source(),
-    'nybb': nybb_source()
+    'nybb': nybb_source(),
+    'elevation': elevation_source()
 }
 
 user_datasets = get_user_datasets()
