@@ -14,15 +14,8 @@ from mapshader.core import render_map
 from mapshader.core import render_geojson
 
 from mapshader.sources import get_user_datasets
-from mapshader.sources import world_countries_source
-from mapshader.sources import world_cities_source
-from mapshader.sources import nybb_source
 
-HERE = path.abspath(path.dirname(__file__))
-FIXTURES_DIR = path.join(HERE, 'fixtures')
-DEFAULT_SOURCES_FUNCS = [world_countries_source,
-                         world_cities_source,
-                         nybb_source]
+from mapshader.tests.data import DEFAULT_SOURCES_FUNCS
 
 
 @pytest.mark.parametrize("source_func", DEFAULT_SOURCES_FUNCS)
@@ -32,7 +25,11 @@ def test_default_to_geojson(source_func):
     assert isinstance(geojson, str)
     data = json.loads(geojson)
     assert isinstance(data, dict)
-    assert data.get('type') == 'FeatureCollection'
+
+    if not source.geometry_type in ('raster', 'line'):
+        assert data.get('type') == 'FeatureCollection'
+    else:
+        assert data
 
 
 @pytest.mark.parametrize("source_func", DEFAULT_SOURCES_FUNCS)
